/**
 *  abstract class at root of shore animal hierarchy
 *  provides functionality for animal ages and when they die
 *  also defines  abstract methods makeMove() and getPrintChar()
 *  field int age - animal's age (private)
 *  field int maxAge - maximun age this animal can attain (private)
 *  @author - Barbara Ryder
 */
public abstract class Animal extends Object
{    private int age; 
     private int maxAge;     
    /**
     *  observer of animal's age
     *  @return int age of Animal
     */
     public int getAge() {return age;}//is this useful?
    /**
     *  mutator of animal's age; can be called by simulation clients
     *  @return int age of animal
     */  
     public void incrAge() { age++; }
    /**
     *  mutator of animal's maximum age; called by descendent classes
     */
     protected void setmaxAge(int age) {maxAge=age;}
    /**
     *  observer of animal's maximum age; can be called by simulation 
     *  clients
     *  @return int maximum age of animal
     */
     public int getmaxAge() 
     {	return maxAge;  }

    /**
     *  function calculates whether or not the animal attains
     *  its maximum age on this time step; if so then it dies; if not
     *  it survives
     *  @return true if animal dies on this time step, otherwise false
     */
     public boolean timeToDie() { 
	 if (age >= maxAge) return true; else return false;
     }

    /**
     *   function returns printable character representation of animal
     *   @return char print character for animal
     */ 
     abstract public char getPrintChar();


    /**
     *  function returns new Location (position) to move to for an animal;
     *  if there is nowhere legal to move, then the function returns null;
     *  if an animal is sessile, then the function returns null;
     *  @return Location for move
     */
     abstract public Location makeMove(Location l, World w);

    /**
     *  constructor of animal objects initializes age to zero
     */
     Animal() { age = 0;}
}
