/**
 * This class provides encapsulation  a neighborhood in the simulation
 * A neighborhood of a point is like a tic-tac-toe board with the point in 
 * in the middle. 
 * Each Direction object has two fields, x and y, 
 * each of which can be set to -1, 0, +1. 
 *
 * @author Matthew Arnold, extended by Barbara Ryder
 */
public class Direction
{
  private int x;
  private int y;

    /**
     *  Observer for x value of a Direction object
     *
     *  @return int x
     *
     */
  public int getX() { return x; }

    /**
     *  Observer for y value of a Direction object
     *
     *  @return int y
     *
     */
  public int getY() { return y; }

    /**
     *  Mutator for x value of a Direction object
     *
     *  @param ix is value with which to set x value 
     */
  public void setX(int ix) { x = ix; }

    /**
     *  Mutator for y value of a Direction object
     *
     *  @param iy is value with which to set y value 
     */
  public void setY(int iy) { y = iy; }

    /**
     *  constructor for an initialized Direction object
     *
     *  @param ix - for x direction
     *  @param iy - for y direction
     *
     */
  public Direction (int ix,int iy)
    {
      x = ix;
      y = iy;
    }

     /**
     *  constructor for an uninitialized Direction object
     *
     */
  public Direction()
    {
      // Leave direction as (0,0)
	x = 0;
	y = 0;
    }

     /**
     * checks for (0,0) direction
     *
     * @return true if this is (0,0), false otherwise.
     *
     */
  public boolean isNull() 
    {
      if (x==0 && y==0) 
	return true;
      else
	return false;
    }

     /**
     * 
     * checks if a proposed direction is the same as an E-W or W-E current  
     *
     * @return true if this is with the direction of the current, 
     * false otherwise
     * @param ll is current location
     * @param myWorld is current simulation object
     *
     */
    public boolean isWithCurrent(Location ll, World myWorld)
    {   //function tests new location versus old location
        //compares x co-ordinates to check if move is with the current
        //or not;
	int curr = myWorld.getCurrent();
	Location newLoc = ll.plus(this);
	int diff = newLoc.getX() - ll.getX();
	if (diff > 0 && curr == 1) return true;
        else if (diff < 0 && curr == -1) return true;
	else return false;
    }

     /**
     * 
     * scales the components of a direction object by its parameter
     *
     * @param i is scale factor
     *
     * @return a new Direction
     *
     */
    public Direction incrDirn(int i)
    {  return new Direction(getX()*i,getY()*i);
    }
}
