/**
 *  abstract class which sets the maximum age of all fast moving animals,
 *  defines makeMove() and laysEggs() for all fast moving animals
 *  currently Crab, Shrimp, Starfish
 */
public abstract class FastMover extends MovingAnimal implements ReproducesbyEggs
{/**
 *   constructor sets maximum age field in ancestor class;
 *   fast movers live to be the oldest animals
 */
    FastMover(){ setmaxAge(10);  }
    /**
     *  abstract method forces all actual animal descendent classes
     *  of FastMover to define a method that returns a new animal of 
     *  their kind;  allows laysEggs() to be inherited by all these classes
     *  @return FastMover reference to a new object of a descendent class
     */
    abstract public FastMover getNewFastMover();
    /**
     *  method implements movement for all fast moving animals
     *  first a direction is picked and then multiplied by the 
     *  scale factor (speed field) of this animal; this Location
     *  is checked to see if it is free. if not, a new direction is
     *  tried; if so, then that Location is returned
     *  @return a free Location to which the animal should move
     */
    public Location makeMove(Location myLoc, World myWorld)
    {
	int t; Location there;
	
       for (t = 8; t>0; t--)
       { Direction d = getDirection();
	 //
	 // try to move from current location to a location speed 
	 // steps away in direction d; check if that site is already occupied
	 //
         there = myLoc.plus(d.incrDirn(getSpeed()));
         // need to try another direction
	 if (myWorld.whatIs(there) != null) continue; 
	 else return  there;
       }
	 return null;//animal can't move this time step
     }

    /**
     *  method uses random number generator to select how many
     *  offspring of fast moving animal survive; current settings are
     *  60% 0 survivors, 30% 1 survivor, 10$ 2 survivors.
     *  @return Set of fast moving animals of one descendent class;
     *  an empty Set indicates no survivors
     */
      public Set laysEggs()
      {
	 Set s = new Set();
	 double num = Math.random();
	 int eggs;
	 if (num <.6) eggs = 0;
 	 else if (num < .9) eggs = 1;
	 else eggs = 2;

	 int i;
	 FastMover fm;
	 for (i = 1; i <= eggs; i++)
	 { fm = getNewFastMover();
	   s.addTo(fm);
	 }
	 return s;
      }
}


