
/**
 * This class is used for implementing a queue as a linked list.
 *
 * @author Rutgers-CS111
 */ 
public class LinkedList extends Object {

private LinkedListNode first;
private LinkedListNode last;

  
  
  /**
   * Create an new LinkedList, initially empty.
   */
  public LinkedList() {
    first  = null;
    last   = null;
  }





  /**
   * Add an Object to the end of the LinkedList
   *
   * @param o  The Object to be added
   */
  public void addTo(Object o)
  {
    LinkedListNode newnode = new LinkedListNode(o);

    if(first == null)
      {
	first  = newnode;
	last   = newnode;
      }
    else
      {
	last.setNext(newnode);
	last = newnode;
      }
  }





  /**
   * Remove an Object from the front of the LinkedList.  This returns
   * null if the list is empty.
   *
   * @return  The Object at the front of the LinkedList.
   */
  public Object removeFrom()
  {
    Object d;

    if(first == null)   // The list is empty
      {
	d = null;
      }
    else if (first.getNext() == null) // The list has one element
      {
	d = first.getData();

	first = null;
	last  = null;
      }
    else
      {
	d = first.getData();

	first = first.getNext();
      }
	
    return(d);
  }





  /**
   * Print out a string representation of a LinkedList.  This assumes
   * that every Object implements the toString() method.
   *
   * @return A String representation of the LinkedList
   */
  public String toString()
  {
    String out = "";

    LinkedListEnumeration lle = new LinkedListEnumeration(this);
    Object d;

    while(lle.hasMoreElements())
      {
	d = lle.nextElement();
	
	out = out + "\t" + d.toString();

	if(lle.hasMoreElements())
	{
	  out = out + "\n";	
	}		
      }

    return(out);
  }



  /**
   * See if a LinkedList is empty.
   *
   * @return true if the LinkedList is empty, false otherwise.
   */
  public boolean isEmpty()
  {
    if(first == null)
      {
	return(true);
      }
    else
      {
	return(false);
      }
  }



  /**
   * Get the first LinkedListNode on a LinkedList
   *
   * @return  The first LinkedListNode
   */
  protected LinkedListNode getFirstNode()
  {
    return(first);
  }
}









