
/**
 * Implements a Set using a cs111.LinkedList.  A Set is an 
 * unordered collection of unique objects.
 */
public class Set extends Object implements Cloneable
{
  private LinkedList list;


  /**
   * Create a new Set, initially empty.
   */
  public Set()
  {
    list = new LinkedList();
  }


  /**
   * Create a new Set with the same elements as in the given set.
   *
   * @param s the Set to take the elements from
   */
  public Set(Set s)
  {
    Set newset = (Set)s.clone();

    this.list = newset.list;
  }



  
  /**
   * See if the Set is empty.
   *
   * @return true if the list is empty, false otherwise
   */
  public boolean isEmpty()
  {
    return(list.isEmpty());
  }

  /**
   * Add an object to the Set.  This will print a warning if the element
   * is already in the Set.
   *
   * @param o The Object to be added.
   */
  public void addTo(Object o)
  {
    if(! this.memberOf(o))
      {
	list.addTo(o);
      }
    else
      {
	System.err.println("Warning.  Attempt made to add element in set to the set.");
      }
  }


  /**
   * See if an Object is a member of a Set
   * @param o is object whose membership is tested
   * @return true if the Object is a member of the Set, false otherwise
   */
  public boolean memberOf(Object o)
  {
    LinkedListEnumeration lle = new LinkedListEnumeration(list);
    Object t;
    
    while(lle.hasMoreElements())
      {
	t = lle.nextElement();

	if(t.equals(o))
	  {
	    return(true);
	  }
      }

    return(false);
  }

  
  /**
   * Return a String representation of a Set.  This assumes every element
   * in the Set implements the toString() method.
   * @return The Set as a String
   */
  public String toString()
  {
    if(this.isEmpty())
      {
	return("\tSet is empty");
      }
    else
      {
	return(list.toString());
      }
  }

  /**
   * Return the LinkedList which holds this Set
   *
   * @return the LinkedList which represents this Set.
   */
  protected LinkedList getList()
  {
    return(list);
  }


  /**
   * We will make a clone of a Set.  The clone of the Set will hold
   * "pointers" to the same objects that are in the original set.  
   * Altering an Object in one Set will alter the Object in the other 
   * Set.
   *
   * @returns a clone of the set.
   */
  protected Object clone()
  {
    Set cl = new Set();
    Object o;

    LinkedListEnumeration lle = new LinkedListEnumeration(list);

    while(lle.hasMoreElements())
      {
	o = lle.nextElement();

	cl.addTo(o);
      }
    
    return(cl);
  }

}




